// parsled.cpp main file
//
// Copyright Matthias Gaertner, Schenefeld, Germany, 2001
// You may use this code under the terms of the GPL.
// Updated packages may be found at http://www.rtner.de/parsled/
//

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <parsled.h>
#include <ParSledConfiguration.h>
#include <ParSledExecutor.h>

#ifndef SIMULATION
#include <unistd.h>
#include <signal.h>
#endif


#ifndef DEFAULT_CONFIG_FILE
#define DEFAULT_CONFIG_FILE "/etc/parsled.conf"
#endif

// Global Variables: accessible to signal handler
ParSledConfiguration *sc = NULL;
ParSledExecutor *se = NULL;

/* Signal handler: all led off then exit */
#ifndef SIMULATION
extern "C"
{

void offandexit(int code)
{
    if( se != NULL )
    {
        se->stop();
    }
    delete se;

    const char *pidfile = sc->getGeneral()->getPIDFile();
    if( pidfile != NULL && strlen( pidfile ) > 0 )
    {
        (void)unlink( pidfile );
    }

    delete sc;
    exit(0);
}

}
#endif

void usage( void )
{
    printf(
"parsled -- Parallel & Serial line pushbutton and LED control daemon\n"
"USAGE:\n"
"    parsled [-D] [-h] [config-file]\n"
"\n"
"OPTIONS:\n"
"    -D         do not detach from console\n"
"    -h,--help  show this help\n"
"    --         force next argument to be treated as configuration filename\n"
"    default for config file is /etc/parsled.conf\n"
"\n"
"EXAMPLE:\n"
"    parsled /etc/my-parsled.conf\n"
"\n"
"See the distribution config file for an explanation of available features.\n"
"This program forks itself to the background (unless -D is given).\n"
"It must have permissions to execute the commands listed in the config file.\n"
"It is assumed that this program gets killed during system shutdown only.\n"
"\n"
"Copyright Matthias Gaertner, Schenefeld, Germany, 2002  -  Version 0.5\n"
"This code is based on ideas from Guido Socher's sled (www.linuxfocus.org)\n"
"Extended for parallel port by Volker Gering\n");
}

int main( int argc, char **argv )
{
    FILE *f = NULL;
    int r;
    const char *pszConfigFile = DEFAULT_CONFIG_FILE;
    bool fDoDetach = true;

    do
    {

    while( argc > 1 && argv[1][0] == '-' )
    {
        if( strcmp(argv[1],"--") == 0 )
        {
            argc--;
            argv++;
            break;
        }

        if( strcmp(argv[1],"-h") == 0 || strcmp(argv[1],"--help") == 0 )
        {
            usage();
            exit(1);
        }

        if( strcmp(argv[1],"-D") == 0 )
        {
            fDoDetach = false;
            argc--;
            argv++;
        }
    }

    if( argc > 1 )
    {
        pszConfigFile = argv[1];
    }

    if( (f=fopen(pszConfigFile,"r")) == NULL )
    {
        r = 3;
        break;
    }

    sc = new ParSledConfiguration();
    if( (r=sc->feedConfiguration(f)) != 0 )
    {
        break;
    }

    if( f != NULL )
    {
        fclose(f);
        f = NULL;
    }

    se = new ParSledExecutor();
    if( (r=se->start(sc)) != 0 )
    {
        break;
    }

#ifndef SIMULATION
    if( fDoDetach )
    {
        signal(SIGINT, offandexit);
        signal(SIGTERM, offandexit);

        // Now we fork to background
        pid_t pid;
        if( (pid=fork()) < 0 )
        {
            perror("parsled server fork");
            exit(1);
        }
        else if( pid > 0 )
        {
            const char *pidfile = sc->getGeneral()->getPIDFile();
            if( pidfile != NULL && strlen( pidfile ) > 0 )
            {
                if( (f=fopen( pidfile, "w" )) != NULL )
                {
                    fprintf( f, "%u\n", pid );
                    fclose( f );
                    f = NULL;
                }
            }

            /* parent dies */
            exit(0);
        }
    }
#endif

    if( (r=se->execute()) != 0 )
    {
        break;
    }

    } while( false );

    if( f != NULL )
    {
        fclose(f);
    }

    delete sc;
    delete se;

    return r;
}

/* */
